import re
import sys
import gzip
import os
import time

try:
    from itertools import product
except ImportError:
    #need to define a Cartesian product function for Python versions < 2.6
    #this one is taken straight from the Python documentation
    def product(*args, **kwds):
        pools = map(tuple, args) * kwds.get('repeat', 1)
        result = [[]]
        for pool in pools:
            result = [x+[y] for x in result for y in pool]
        for prod in result:
            yield tuple(prod)

def get_factors(motif_len):
    factor_list = []    
    for aa in range(1, motif_len):
        if motif_len % aa == 0:
            factor_list.append(aa)
    return factor_list

def get_prefix_and_badmotifs(repeat_len):
    prefix = str(repeat_len) + '-mers'
    bad_motifs = set()
    factors = get_factors(repeat_len)
        
    for factor in factors:
        multiplier = repeat_len / factor
        permutations = (''.join(x) for x in product('ACGT', repeat=factor))
        for p in permutations:
            motif = p * multiplier
            if motif not in bad_motifs:
                bad_motifs.add(motif)
            
    return prefix, bad_motifs


def findssrs(filename, repeat_len, min_repeats, flank_len, FASTQ, 
             startnewfile):
    
    time1 = time.asctime()
    
    prefix, bad_motifs = get_prefix_and_badmotifs(repeat_len)

    if filename[-3:] == '.gz':    
        f = gzip.open(filename, 'r')
    else:
        f = open(filename, 'r')

    regex1 = r'(([AGCT]{rep_len})\2{nmin,})'
    regex1 = regex1.replace('nmin', str(min_repeats-1))
    regex1 = regex1.replace('rep_len', str(repeat_len))
    regex = re.compile(regex1)

    outfile = ('SSRs_' + prefix + '_min-repeats=' + str(min_repeats) + 
                '_flank-len=' + str(flank_len) +  '.fasta')
    
    if startnewfile:
        o = open(outfile, 'w')
    else:
        o = open(outfile, 'a')

    nseqs = 0
    nssrs = 0
    ngoodssrs = 0
    for line in f:
        nseqs += 1
        seqname = line.strip()
        seqname = '>' + seqname[1:]
        sequence = f.next().strip()
        sequence = sequence.upper()
        
        if FASTQ:
            blank1 = f.next()
            blank2 = f.next()

        items = []
        matches1 = regex.finditer(sequence)
        for item in matches1:
            if item.group()[:repeat_len] not in bad_motifs:
                items.append(item)
                
        if len(items) > 0:
            nssrs += 1
            keeper = 0
            motifs = []
            spans = []
            for match in items:
                if ((match.start() >= flank_len) and 
                    (match.end() <= (len(sequence) - flank_len))):
                    keeper +=1
                motif = match.group()[:repeat_len]
                motif = motif + '-' + str(len(match.group())/repeat_len)
                motifs.append(motif)
                spans.append(match.span())
                                        
            if keeper == len(items):
                ngoodssrs += 1
                motifsandspans = ""
                for qq in range(len(motifs)):
                    motifsandspans = (motifsandspans + ' ' + motifs[qq] + 
                        '(' + str(spans[qq][0]+1) + ',' + 
                        str(spans[qq][1]) + ')')
                newseqname = seqname + "|" + motifsandspans
                o.write(newseqname + '\n')
                o.write(sequence + '\n')
            else:
                motifsandspans = ""
                for qq in range(len(motifs)):
                    motifsandspans = (motifsandspans + ' ' + motifs[qq] + 
                        '(' + str(spans[qq][0]+1) + ',' + 
                        str(spans[qq][1]) + ')')
                newseqname = seqname + "|" + motifsandspans
                
    f.close()
    o.close()
    
    f = open('analysis_report.txt', 'a')
    f.write('\n\n**Module SSR_search**:\n')
    f.write("Analysis start: " + time1 + '\n')    
    f.write("Search for " + prefix + " motifs with at least " + 
            str(min_repeats) + ' repeats.\n')
    f.write(str(flank_len) + ' bp flanking sequences preserved.\n')
    f.write(str(nseqs) + " sequences examined.\n")
    f.write(str(nssrs) + " sequences contained SSRs.\n")
    f.write(str(ngoodssrs) + 
        " sequences contained SSRs only between the specified " + 
        str(flank_len) + " bp flanking sequences.\n")
    f.write("Saved sequences written to file " + outfile + '\n')
    f.write("Analysis complete:" + time.asctime() + '\n')
    f.close()
    
    
def call_findSSRs_from_CL():
    try:
        filename = sys.argv[1]
        repeat_len = int(sys.argv[2])
        min_repeats = int(sys.argv[3])
        flank_len = int(sys.argv[4])
        
        if filename[-3:] == '.gz':    
            f = gzip.open(filename, 'r')
        else:
            f = open(filename, 'r')
        line = f.readline().strip()
        if line[0] == '@':
            FASTQ = True
        else:
            FASTQ = False
        f.close()
    except:
        sys.exit("""
Error. Check filenames and arguments passed to program.
Program usage is as follows:
    SSR_search.py f1 repeat_len min_repeats flank_len
where
    f1 = complete name and path to file to be analyzed
    repeat_len = integer specifying the desired repeat motif length to look 
        for (di-, tri-, tetranucleotides, etc.)
    min_repeats = integer specifying the minimum number of repeat units 
        that a sequence needs to possess before it will be saved in the 
        analysis output
    flank_len = integer specifying the desired length of flanking sequences.
        An SSR must be detected outside of the flanking sequences in order to
        be recorded in output files. Note that by setting this value to 0, all
        sequences containing SSRs can be identified.
""")   

    path_to_file = os.path.dirname(filename)
    fname = os.path.basename(filename)
    os.chdir(path_to_file)
    startnewfile = True
    findssrs(fname, repeat_len, min_repeats, flank_len, FASTQ, startnewfile)
    

def call_findSSRs_from_func(filename, repeat_len, min_repeats, flank_len,
                            startnewfile):
    if filename[-3:] == '.gz':    
        f = gzip.open(filename, 'r')
    else:
        f = open(filename, 'r')
    line = f.readline().strip()
    if line[0] == '@':
        FASTQ = True
    else:
        FASTQ = False
    f.close()    

    findssrs(filename, repeat_len, min_repeats, flank_len, FASTQ, startnewfile)

       
if __name__ == '__main__':
    call_findSSRs_from_CL()
    





