import sys
import os
import gzip
import time
from string import maketrans
from joinseqs_ext import joinseqs, matchinfo

    
def join_sequences(f1name, f2name, min_overlap, match_ratio_threshold, 
                   max_overlap, keepZipped):

    time1 = time.asctime()
    if f1name[-3:] == '.gz':
        file1 = gzip.open(f1name, 'r')
        file2 = gzip.open(f2name, 'r')
    else:
        file1 = open(f1name, 'r')
        file2 = open(f2name, 'r')

    #need to switch to output_subdir_name
    if keepZipped:
        joinfile = gzip.open('joined_reads_' + os.path.basename(f1name), 'w')
        notjoined1 = gzip.open('unjoined1_' + os.path.basename(f1name), 'w')
        notjoined2 = gzip.open('unjoined2_' + os.path.basename(f2name), 'w')
    else:
        joinfile = open('joined_reads_' + os.path.basename(f1name), 'w')
        notjoined1 = open('unjoined1_' + os.path.basename(f1name), 'w')
        notjoined2 = open('unjoined2_' + os.path.basename(f2name), 'w')

    n_pairs_examined = 0
    n_pairs_joined = 0

    transtable = maketrans('ACGT', 'TGCA')    
    
    for line in file1:
        #read 4 lines from each file and strip non-dispay characters
        line1_1 = line #seq1 name
        line1_2 = file1.next() #seq1 sequence
        line1_3 = file1.next()
        line1_4 = file1.next() #seq1 quality

        line2_1 = file2.next() #seq2 name
        line2_2 = file2.next() #seq2 sequence
        line2_3 = file2.next()
        line2_4 = file2.next() #seq2 quality

        seq1 = line1_2.strip()
        qual1 = line1_4.strip()
        
        #get reverse complement of sequence from line2_2
        seq2 = line2_2.strip()[::-1].translate(transtable)
        #reverse the quality score
        qual2 = line2_4.strip()[::-1] 

        #if sequences are different lengths, pad the shorter ones to
        #make them equal lengths
        lens1 = len(seq1)
        lens2 = len(seq2)
        if lens1 == lens2:
            lendiff = 0
        elif lens1 < lens2:
            lendiff = lens2-lens1
            seq1 = seq1 + ('N' * lendiff)
            qual1 = qual1 + ('#' * lendiff)
        else:
            lendiff = lens1-lens2
            seq2 = seq2 + ('N' * lendiff)
            qual2 = qual2 + ('#' * lendiff)
 
        #bestmatch is a simple externally-defined class (see imports)
        #that stores results of the alignments.
        bestmatch = matchinfo()
        #join the sequences: use compiled function in external module
        joinseqs(seq1, seq2, qual1, qual2, min_overlap, 
                 match_ratio_threshold, max_overlap, bestmatch)

        #record and export the results
        a = bestmatch.matchposition
        matchlen = bestmatch.matchlen
        observed_match_ratio = bestmatch.bestratio
        overlap_seq = bestmatch.overlap_seq
        overlap_qual = bestmatch.overlap_qual
        
        n_pairs_examined += 1
        if (matchlen >= min_overlap and 
            observed_match_ratio <= match_ratio_threshold):
            n_pairs_joined += 1
            joinfile.write(line1_1)
            seqline = seq1[:a] + overlap_seq + seq2[len(overlap_seq):]
            joinfile.write(seqline + '\n')
            joinfile.write(line1_3)
            qualline = qual1[:a] + overlap_qual + qual2[len(overlap_seq):]
            joinfile.write(qualline + '\n')
        else:        
            notjoined1.write(line1_1)
            notjoined1.write(line1_2)
            notjoined1.write(line1_3)
            notjoined1.write(line1_4)

            notjoined2.write(line2_1)
            notjoined2.write(line2_2)
            notjoined2.write(line2_3)
            notjoined2.write(line2_4)
        del(bestmatch)
            
    file1.close()
    file2.close()
    joinfile.close()
    notjoined1.close()
    notjoined2.close()
    
    f = open('analysis_report.txt', 'a')
    f.write('\n\n**Module joinseqs**:\n')
    f.write("Analysis start: " + time1 + '\n')    
    reportstr = str(n_pairs_examined) + ' pairs examined.\n'
    reportstr += str(n_pairs_joined) + ' pairs successfully joined.\n'
    reportstr += 'Parameters used:\n'
    reportstr += ('min_olap = ' + str(min_overlap) + '\n')
    reportstr += ('mr = ' + str(match_ratio_threshold) + '\n')
    reportstr += ('max_olap = ' + str(max_overlap) + '\n')
    f.write(reportstr)
    f.write("Analysis complete: " + time.asctime() + '\n')    
    f.close()


def call_joinseqs_from_cl():
    arguments = sys.argv
    try:
        file1 = arguments[1]                       #file1name
        file2 = arguments[2]                       #file2name
        min_overlap = int(arguments[3])             #min_overlap
        match_ratio_threshold = float(arguments[4]) #match_ratio_threshold
        max_overlap = int(arguments[5])             #max_overlap
        keepZipped = arguments[6]
        if keepZipped == 'T':
            keepZipped = True
        else:
            keepZipped = False
    except:
        sys.exit("""
Error. Check filenames and arguments passed to program.
Program usage is as follows:
    joinseqs.py f1 f2 min_olap mr max_olap keepZipped
where
    f1 = complete name and path to file 1
    f2 = complete name and path to file 2
    min_olap = smallest sequence overlap to be be evaluated (try using 10)
    mr = the mismatch ratio threshold defining the maximum proportion of 
        mismatches allowed within the overlap between two sequences (try 
        using 0.25)
    max_olap = maximum sequence overlap beyond which a penalized mr is 
        calculated (try using 70)
    keepZipped = T or F, corresponding to True or False. If True, then output
        files will be written as gzipped archives to save disk space
        
The alignment algorithm is based on: 
Magoc and Salzberg (2011) FLASH: Fast length adjustment of short reads to 
improve genome assemblies. Bioinformatics 27:2957-2963.
""")   

        
    f1name = os.path.basename(file1)
    f2name = os.path.basename(file2)
    path_to_files = os.path.dirname(file1)

    os.chdir(path_to_files)

    join_sequences(f1name, f2name, min_overlap, match_ratio_threshold, 
                   max_overlap, keepZipped)

    os.chdir('..')

def call_joinseqs_from_func(f1name, f2name, min_overlap, 
                            match_ratio_threshold, max_overlap, keepZipped):
    
        if keepZipped == 'T':
            keepZipped = True
        else:
            keepZipped = False
            
        join_sequences(f1name, f2name, min_overlap, match_ratio_threshold, 
                   max_overlap, keepZipped)



if __name__ == '__main__':
    call_joinseqs_from_cl()

    
